/*-
 * Copyright 2022 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.quality.adapter;

import javax.xml.namespace.QName;

import org.xmlobjects.builder.ObjectBuildException;
import org.xmlobjects.builder.ObjectBuilder;
import org.xmlobjects.gml.adapter.geometry.DirectPositionAdapter;
import org.xmlobjects.serializer.ObjectSerializeException;
import org.xmlobjects.serializer.ObjectSerializer;
import org.xmlobjects.stream.XMLReadException;
import org.xmlobjects.stream.XMLReader;
import org.xmlobjects.stream.XMLWriteException;
import org.xmlobjects.stream.XMLWriter;
import org.xmlobjects.xml.Attributes;
import org.xmlobjects.xml.Element;
import org.xmlobjects.xml.Namespaces;

import de.hft.stuttgart.quality.QualityADEModule;
import de.hft.stuttgart.quality.model.types.Edge;

public class EdgeAdapter implements ObjectBuilder<Edge>, ObjectSerializer<Edge> {

	@Override
	public Edge createObject(QName name, Object parent) throws ObjectBuildException {
		return new Edge();
	}

	@Override
	public void buildChildObject(Edge object, QName name, Attributes attributes, XMLReader reader)
			throws ObjectBuildException, XMLReadException {
		if (!QualityADEModule.NAMESPACE_URI.equals(name.getNamespaceURI())) {
			return;
		}
		switch (name.getLocalPart()) {
		case "from" -> object.setFrom(reader.getObjectUsingBuilder(DirectPositionAdapter.class));
		case "to" -> object.setTo(reader.getObjectUsingBuilder(DirectPositionAdapter.class));
		default -> throw new IllegalStateException("Cannot handle name " + name + " when building edge element");
		}
	}

	@Override
	public void writeChildElements(Edge object, Namespaces namespaces, XMLWriter writer)
			throws ObjectSerializeException, XMLWriteException {
		if (object.getFrom() != null) {
			writer.writeElementUsingSerializer(Element.of(QualityADEModule.NAMESPACE_URI, "from"), object.getFrom(),
					DirectPositionAdapter.class, namespaces);
		}
		if (object.getTo() != null) {
			writer.writeElementUsingSerializer(Element.of(QualityADEModule.NAMESPACE_URI, "to"), object.getTo(),
					DirectPositionAdapter.class, namespaces);
		}
	}

}
